@@ -9,7 +9,7 @@
 
 #include <pika/barrier.hpp>
 #include <pika/concurrency/detail/contiguous_index_queue.hpp>
-#include <pika/future.hpp>
+#include <pika/execution.hpp>
 #include <pika/init.hpp>
 #include <pika/program_options.hpp>
 #include <pika/testing.hpp>
@@ -17,12 +17,17 @@
 #include <algorithm>
 #include <cstddef>
 #include <cstdint>
+#include <cstdlib>
 #include <functional>
 #include <iterator>
 #include <optional>
 #include <random>
+#include <utility>
 #include <vector>
 
+namespace ex = pika::execution::experimental;
+namespace tt = pika::this_thread::experimental;
+
 unsigned int seed = std::random_device{}();
 
 void test_basic()
@@ -127,24 +132,29 @@ void test_concurrent_worker(pop_mode m, std::size_t thread_index, pika::barrier<
 void test_concurrent(pop_mode m)
 {
     std::uint32_t first = 33;
+#if defined(PIKA_HAVE_VALGRIND)
+    std::uint32_t last = 5432;
+#else
     std::uint32_t last = 1372195;
+#endif
     pika::concurrency::detail::contiguous_index_queue<> q{first, last};
 
     std::size_t const num_threads = pika::get_num_worker_threads();
     // This test should be run on at least two worker threads.
     PIKA_TEST_LTE(std::size_t(2), num_threads);
-    std::vector<pika::future<void>> fs;
+    std::vector<ex::unique_any_sender<>> senders;
     std::vector<std::vector<std::uint32_t>> popped_indices(num_threads);
-    fs.reserve(num_threads);
+    senders.reserve(num_threads);
     pika::barrier<> b(num_threads);
 
     for (std::size_t i = 0; i < num_threads; ++i)
     {
-        fs.push_back(pika::async(
-            test_concurrent_worker, m, i, std::ref(b), std::ref(q), std::ref(popped_indices[i])));
+        senders.push_back(ex::transfer_just(ex::thread_pool_scheduler{}, m, i, std::ref(b),
+                              std::ref(q), std::ref(popped_indices[i])) |
+            ex::then(test_concurrent_worker));
     }
 
-    pika::wait_all(fs);
+    tt::sync_wait(ex::when_all_vector(std::move(senders)));
 
     // There should be no indices left in the queue at this point.
     PIKA_TEST(q.empty());
@@ -171,10 +181,14 @@ void test_concurrent(pop_mode m)
         ++curr_expected;
     }
 
-    // We expect at least two threads to have popped indices concurrently.
-    // There is a small chance of false positives here (resulting from big
-    // delays in starting threads).
+    // We expect at least two threads to have popped indices concurrently.  There is a small chance
+    // of false positives here (resulting from big delays in starting threads).
+    //
+    // With valgrind enabled in non-debug builds the false positive rate is too high, so we skip
+    // this check.
+#if !defined(PIKA_HAVE_VALGRIND) || defined(PIKA_DEBUG)
     PIKA_TEST_LTE(std::size_t(2), num_nonzero_indices_popped);
+#endif
 }
 
 int pika_main(pika::program_options::variables_map& vm)
@@ -185,7 +199,9 @@ int pika_main(pika::program_options::variables_map& vm)
     test_concurrent(pop_mode::left);
     test_concurrent(pop_mode::right);
     test_concurrent(pop_mode::random);
-    return pika::finalize();
+
+    pika::finalize();
+    return EXIT_SUCCESS;
 }
 
 int main(int argc, char** argv)
