@@ -297,12 +297,8 @@ namespace pika::ensure_started_detail {
 
                 if (continuation)
                 {
-                    // We move the continuation to a local variable to release it once the
-                    // continuation has been called. We cannot call reset on continuation after the
-                    // invocation because the shared state may already be released by the
-                    // continuation itself.
-                    auto continuation_local = PIKA_MOVE(continuation.value());
-                    continuation_local();
+                    continuation.value()();
+                    continuation.reset();
                 }
             }
 
@@ -446,8 +442,8 @@ namespace pika::ensure_started_detail {
         {
             static_assert(sizeof(Receiver) == 0,
                 "Are you missing a std::move? The ensure_started sender is not copyable and thus "
-                "not l-value connectable. Make sure you are passing an r-value reference of the "
-                "sender.");
+                "not l-value connectable. Make sure you are passing a non-const r-value reference "
+                "of the sender.");
         }
     };
 
