@@ -11,12 +11,9 @@
 #include <pika/command_line_handling/command_line_handling.hpp>
 #include <pika/coroutines/detail/context_impl.hpp>
 #include <pika/detail/filesystem.hpp>
-#include <pika/execution/detail/execution_parameter_callbacks.hpp>
 #include <pika/execution_base/detail/spinlock_deadlock_detection.hpp>
-#include <pika/executors/exception_list.hpp>
 #include <pika/functional/bind_front.hpp>
 #include <pika/functional/function.hpp>
-#include <pika/futures/detail/future_data.hpp>
 #include <pika/init_runtime/detail/init_logging.hpp>
 #include <pika/init_runtime/init_runtime.hpp>
 #include <pika/lock_registration/detail/register_locks.hpp>
@@ -30,7 +27,6 @@
 #include <pika/runtime/config_entry.hpp>
 #include <pika/runtime/custom_exception_info.hpp>
 #include <pika/runtime/debugging.hpp>
-#include <pika/runtime/get_locality_id.hpp>
 #include <pika/runtime/runtime.hpp>
 #include <pika/runtime/runtime_handlers.hpp>
 #include <pika/runtime/shutdown_function.hpp>
@@ -38,6 +34,7 @@
 #include <pika/string_util/classification.hpp>
 #include <pika/string_util/split.hpp>
 #include <pika/threading/thread.hpp>
+#include <pika/threading_base/detail/get_default_pool.hpp>
 #include <pika/type_support/pack.hpp>
 #include <pika/type_support/unused.hpp>
 #include <pika/util/get_entry_as.hpp>
@@ -110,174 +107,7 @@ namespace pika {
             // Invoke custom startup functions
             return f(static_cast<int>(argcount), argv.data());
         }
-    }    // namespace detail
 
-    // Print stack trace and exit.
-    int init(std::function<int(pika::program_options::variables_map&)> f, int argc,
-        const char* const* argv, init_params const& params)
-    {
-        return detail::init_start_impl(PIKA_MOVE(f), argc, argv, params, true);
-    }
-
-    int init(std::function<int(int, char**)> f, int argc, const char* const* argv,
-        init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f =
-            pika::util::detail::bind_back(pika::detail::init_helper, f);
-        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
-    }
-
-    int init(std::function<int()> f, int argc, const char* const* argv, init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f =
-            pika::util::detail::bind(f);
-        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
-    }
-
-    int init(std::nullptr_t, int argc, const char* const* argv, init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f;
-        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
-    }
-
-    bool start(std::function<int(pika::program_options::variables_map&)> f, int argc,
-        const char* const* argv, init_params const& params)
-    {
-        return 0 == detail::init_start_impl(PIKA_MOVE(f), argc, argv, params, false);
-    }
-
-    bool start(std::function<int(int, char**)> f, int argc, const char* const* argv,
-        init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f =
-            pika::util::detail::bind_back(pika::detail::init_helper, f);
-        return 0 == detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false);
-    }
-
-    bool start(std::function<int()> f, int argc, const char* const* argv, init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f =
-            pika::util::detail::bind(f);
-        return 0 == detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false);
-    }
-
-    bool start(std::nullptr_t, int argc, const char* const* argv, init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f;
-        return 0 == detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false);
-    }
-
-    bool start(int argc, const char* const* argv, init_params const& params)
-    {
-        util::detail::function<int(pika::program_options::variables_map&)> main_f;
-        return 0 == detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false);
-    }
-
-    int finalize(error_code& ec)
-    {
-        if (!is_running())
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::finalize",
-                "the runtime system is not active (did you already call finalize?)");
-            return -1;
-        }
-
-        if (&ec != &throws) ec = make_success_code();
-
-        runtime* rt = get_runtime_ptr();
-        if (nullptr == rt)
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::finalize",
-                "the runtime system is not active (did you already call pika::stop?)");
-            return -1;
-        }
-
-        rt->finalize();
-
-        return 0;
-    }
-
-    int stop(error_code& ec)
-    {
-        if (threads::detail::get_self_ptr())
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::stop",
-                "this function cannot be called from a pika thread");
-            return -1;
-        }
-
-        std::unique_ptr<runtime> rt(get_runtime_ptr());    // take ownership!
-        if (nullptr == rt.get())
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::stop",
-                "the runtime system is not active (did you already call pika::stop?)");
-            return -1;
-        }
-
-        int result = rt->wait();
-
-        rt->stop();
-        rt->rethrow_exception();
-
-        return result;
-    }
-
-    int wait(error_code& ec)
-    {
-        runtime* rt = get_runtime_ptr();
-        if (nullptr == rt)
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::wait",
-                "the runtime system is not active (did you already call pika::stop?)");
-            return -1;
-        }
-
-        rt->get_thread_manager().wait();
-
-        return 0;
-    }
-
-    int suspend(error_code& ec)
-    {
-        if (threads::detail::get_self_ptr())
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::suspend",
-                "this function cannot be called from a pika thread");
-            return -1;
-        }
-
-        runtime* rt = get_runtime_ptr();
-        if (nullptr == rt)
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::suspend",
-                "the runtime system is not active (did you already call pika::stop?)");
-            return -1;
-        }
-
-        return rt->suspend();
-    }
-
-    int resume(error_code& ec)
-    {
-        if (threads::detail::get_self_ptr())
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::resume",
-                "this function cannot be called from a pika thread");
-            return -1;
-        }
-
-        runtime* rt = get_runtime_ptr();
-        if (nullptr == rt)
-        {
-            PIKA_THROWS_IF(ec, pika::error::invalid_status, "pika::resume",
-                "the runtime system is not active (did you already call pika::stop?)");
-            return -1;
-        }
-
-        return rt->resume();
-    }
-
-    namespace detail {
         void activate_global_options(detail::command_line_handling& cmdline)
         {
 #if defined(__linux) || defined(linux) || defined(__linux__) || defined(__FreeBSD__)
@@ -311,6 +141,24 @@ namespace pika {
 #endif
         }
 
+        struct dump_config
+        {
+            explicit dump_config(pika::runtime const& rt)
+              : rt_(std::cref(rt))
+            {
+            }
+
+            void operator()() const
+            {
+                std::cout << "Configuration after runtime start:\n";
+                std::cout << "----------------------------------\n";
+                rt_.get().get_config().dump(0, std::cout);
+                std::cout << "----------------------------------\n";
+            }
+
+            std::reference_wrapper<pika::runtime const> rt_;
+        };
+
         ///////////////////////////////////////////////////////////////////////
         void add_startup_functions(pika::runtime& rt, pika::program_options::variables_map& vm,
             startup_function_type startup, shutdown_function_type shutdown)
@@ -385,8 +233,7 @@ namespace pika {
             start(*rt, cfg.pika_main_f_, cfg.vm_, PIKA_MOVE(startup), PIKA_MOVE(shutdown));
 
             // pointer to runtime is stored in TLS
-            pika::runtime* p = rt.release();
-            (void) p;
+            [[maybe_unused]] pika::runtime* p = rt.release();
 
             return 0;
         }
@@ -401,9 +248,6 @@ namespace pika {
             pika::detail::set_pre_exception_handler(&pika::detail::pre_exception_handler);
             pika::set_thread_termination_handler(
                 [](std::exception_ptr const& e) { report_error(e); });
-            pika::lcos::detail::set_run_on_completed_error_handler([](std::exception_ptr const& e) {
-                pika::detail::report_exception_and_terminate(e);
-            });
             pika::detail::set_get_full_build_string(&pika::full_build_string);
 #if defined(PIKA_HAVE_VERIFY_LOCKS)
             pika::util::set_registered_locks_error_handler(
@@ -417,10 +261,6 @@ namespace pika {
             }
 
             pika::threads::detail::set_get_default_pool(&pika::detail::get_default_pool);
-            pika::threads::detail::set_get_locality_id(&get_locality_id);
-            pika::parallel::execution::detail::set_get_pu_mask(&pika::detail::get_pu_mask);
-            pika::parallel::execution::detail::set_get_os_thread_count(
-                []() { return pika::get_os_thread_count(); });
 
 #if defined(__bgq__) || defined(__bgqion__)
             unsetenv("LANG");
@@ -440,104 +280,254 @@ namespace pika {
 #endif
         }
 
-        // make sure the runtime system is not active yet
-        int ensure_no_runtime_is_up()
-        {
-            // make sure the runtime system is not active yet
-            if (get_runtime_ptr() != nullptr)
-            {
-                std::cerr
-                    << "pika::init: can't initialize runtime system more than once! Exiting...\n";
-                return -1;
-            }
-            return 0;
-        }
-
         ///////////////////////////////////////////////////////////////////////
         int run_or_start(
             util::detail::function<int(pika::program_options::variables_map& vm)> const& f,
             int argc, const char* const* argv, init_params const& params, bool blocking)
         {
-            int result = 0;
-            try
+            if (get_runtime_ptr() != nullptr)
             {
-                if ((result = ensure_no_runtime_is_up()) != 0) { return result; }
+                PIKA_THROW_EXCEPTION(
+                    pika::error::invalid_status, "pika::start/init", "runtime already initialized");
+            }
 
-                pika::detail::command_line_handling cmdline{
-                    pika::util::runtime_configuration(argv[0]), params.cfg, f};
+            pika::detail::command_line_handling cmdline{
+                pika::util::runtime_configuration(argv[0]), params.cfg, f};
 
-                // scope exception handling to resource partitioner initialization
-                // any exception thrown during run_or_start below are handled
-                // separately
-                try
-                {
-                    result = cmdline.call(params.desc_cmdline, argc, argv);
+            auto cmdline_result = cmdline.call(params.desc_cmdline, argc, argv);
 
-                    pika::detail::affinity_data affinity_data{};
-                    affinity_data.init(pika::detail::get_entry_as<std::size_t>(
-                                           cmdline.rtcfg_, "pika.os_threads", 0),
-                        pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.cores", 0),
-                        pika::detail::get_entry_as<std::size_t>(
-                            cmdline.rtcfg_, "pika.pu_offset", 0),
-                        pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.pu_step", 0),
-                        0, cmdline.rtcfg_.get_entry("pika.affinity", ""),
-                        cmdline.rtcfg_.get_entry("pika.bind", ""),
-                        !pika::detail::get_entry_as<bool>(
-                            cmdline.rtcfg_, "pika.ignore_process_mask", false));
+            pika::detail::affinity_data affinity_data{};
+            affinity_data.init(
+                pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.os_threads", 0),
+                pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.cores", 0),
+                pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.pu_offset", 0),
+                pika::detail::get_entry_as<std::size_t>(cmdline.rtcfg_, "pika.pu_step", 0), 0,
+                cmdline.rtcfg_.get_entry("pika.affinity", ""),
+                cmdline.rtcfg_.get_entry("pika.bind", ""),
+                !pika::detail::get_entry_as<bool>(
+                    cmdline.rtcfg_, "pika.ignore_process_mask", false));
 
-                    pika::resource::partitioner rp = pika::resource::detail::make_partitioner(
-                        params.rp_mode, cmdline.rtcfg_, affinity_data);
+            pika::resource::partitioner rp = pika::resource::detail::make_partitioner(
+                params.rp_mode, cmdline.rtcfg_, affinity_data);
 
-                    activate_global_options(cmdline);
+            activate_global_options(cmdline);
 
-                    init_environment(cmdline);
+            init_environment(cmdline);
 
-                    // check whether pika should be exited at this point
-                    // (parse_result is returning a result > 0, if the program options
-                    // contain --pika:help or --pika:version, on error result is < 0)
-                    if (result != 0)
-                    {
-                        if (result > 0) result = 0;
-                        return result;
-                    }
+            switch (cmdline_result)
+            {
+            case command_line_handling_result::success: break;
+            case command_line_handling_result::exit: return 0;
+            }
 
-                    // If thread_pools initialization in user main
-                    if (params.rp_callback) { params.rp_callback(rp, cmdline.vm_); }
+            // If thread_pools initialization in user main
+            if (params.rp_callback) { params.rp_callback(rp, cmdline.vm_); }
 
-                    // Setup all internal parameters of the resource_partitioner
-                    rp.configure_pools();
-                }
-                catch (pika::exception const& e)
-                {
-                    std::cerr << "pika::init: pika::exception caught: " << pika::get_error_what(e)
-                              << "\n";
-                    return -1;
-                }
+            // Setup all internal parameters of the resource_partitioner
+            rp.configure_pools();
 
 #if defined(PIKA_HAVE_HIP)
-                LPROGRESS_ << "run_local: initialize HIP";
-                whip::check_error(hipInit(0));
+            LPROGRESS_ << "run_local: initialize HIP";
+            whip::check_error(hipInit(0));
 #endif
 
-                // Initialize and start the pika runtime.
-                LPROGRESS_ << "run_local: create runtime";
+            // Initialize and start the pika runtime.
+            LPROGRESS_ << "run_local: create runtime";
 
-                // Build and configure this runtime instance.
-                std::unique_ptr<pika::runtime> rt;
+            // Build and configure this runtime instance.
+            std::unique_ptr<pika::runtime> rt;
 
-                // Command line handling should have updated this by now.
-                LPROGRESS_ << "creating local runtime";
-                rt.reset(new pika::runtime(cmdline.rtcfg_, true));
+            // Command line handling should have updated this by now.
+            LPROGRESS_ << "creating local runtime";
+            rt.reset(new pika::runtime(cmdline.rtcfg_, true));
 
-                result = run_or_start(blocking, PIKA_MOVE(rt), cmdline, PIKA_MOVE(params.startup),
-                    PIKA_MOVE(params.shutdown));
-            }
-            catch (pika::detail::command_line_error const& e)
+            return run_or_start(blocking, PIKA_MOVE(rt), cmdline, PIKA_MOVE(params.startup),
+                PIKA_MOVE(params.shutdown));
+        }
+
+        int init_start_impl(util::detail::function<int(pika::program_options::variables_map&)> f,
+            int argc, const char* const* argv, init_params const& params, bool blocking)
+        {
+            if (argc == 0 || argv == nullptr)
             {
-                std::cerr << "pika::init: std::exception caught: " << e.what() << "\n";
-                return -1;
+                argc = dummy_argc;
+                argv = dummy_argv;
             }
-            return result;
+
+#if defined(__FreeBSD__)
+            freebsd_environ = environ;
+#endif
+            // set a handler for std::abort
+            [[maybe_unused]] auto signal_handler = std::signal(SIGABRT, pika::detail::on_abort);
+            [[maybe_unused]] auto exit_result = std::atexit(pika::detail::on_exit);
+#if defined(PIKA_HAVE_CXX11_STD_QUICK_EXIT)
+            [[maybe_unused]] auto quick_exit_result = std::at_quick_exit(pika::detail::on_exit);
+#endif
+            return run_or_start(f, argc, argv, params, blocking);
         }
     }    // namespace detail
+
+    int init(std::function<int(pika::program_options::variables_map&)> f, int argc,
+        const char* const* argv, init_params const& params)
+    {
+        return detail::init_start_impl(PIKA_MOVE(f), argc, argv, params, true);
+    }
+
+    int init(std::function<int(int, char**)> f, int argc, const char* const* argv,
+        init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f =
+            pika::util::detail::bind_back(pika::detail::init_helper, f);
+        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
+    }
+
+    int init(std::function<int()> f, int argc, const char* const* argv, init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f =
+            pika::util::detail::bind(f);
+        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
+    }
+
+    int init(std::nullptr_t, int argc, const char* const* argv, init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f;
+        return detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, true);
+    }
+
+    void start(std::function<int(pika::program_options::variables_map&)> f, int argc,
+        const char* const* argv, init_params const& params)
+    {
+        if (detail::init_start_impl(PIKA_MOVE(f), argc, argv, params, false) != 0)
+        {
+            PIKA_UNREACHABLE;
+        }
+    }
+
+    void start(std::function<int(int, char**)> f, int argc, const char* const* argv,
+        init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f =
+            pika::util::detail::bind_back(pika::detail::init_helper, f);
+        if (detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false) != 0)
+        {
+            PIKA_UNREACHABLE;
+        }
+    }
+
+    void start(std::function<int()> f, int argc, const char* const* argv, init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f =
+            pika::util::detail::bind(f);
+        if (detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false) != 0)
+        {
+            PIKA_UNREACHABLE;
+        }
+    }
+
+    void start(std::nullptr_t, int argc, const char* const* argv, init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f;
+        if (detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false) != 0)
+        {
+            PIKA_UNREACHABLE;
+        }
+    }
+
+    void start(int argc, const char* const* argv, init_params const& params)
+    {
+        util::detail::function<int(pika::program_options::variables_map&)> main_f;
+        if (detail::init_start_impl(PIKA_MOVE(main_f), argc, argv, params, false) != 0)
+        {
+            PIKA_UNREACHABLE;
+        }
+    }
+
+    void finalize()
+    {
+        if (!is_running())
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::finalize",
+                "the runtime system is not active (did you already call finalize?)");
+        }
+
+        runtime* rt = get_runtime_ptr();
+        if (nullptr == rt)
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::finalize",
+                "the runtime system is not active (did you already call pika::stop?)");
+        }
+
+        rt->finalize();
+    }
+
+    int stop()
+    {
+        if (threads::detail::get_self_ptr())
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::stop",
+                "this function cannot be called from a pika thread");
+        }
+
+        std::unique_ptr<runtime> rt(get_runtime_ptr());    // take ownership!
+        if (nullptr == rt.get())
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::stop",
+                "the runtime system is not active (did you already call pika::stop?)");
+        }
+
+        int result = rt->wait();
+
+        rt->stop();
+        rt->rethrow_exception();
+
+        return result;
+    }
+
+    void wait()
+    {
+        runtime* rt = get_runtime_ptr();
+        if (nullptr == rt)
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::wait",
+                "the runtime system is not active (did you already call pika::stop?)");
+        }
+
+        rt->get_thread_manager().wait();
+    }
+
+    void suspend()
+    {
+        if (threads::detail::get_self_ptr())
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::suspend",
+                "this function cannot be called from a pika thread");
+        }
+
+        runtime* rt = get_runtime_ptr();
+        if (nullptr == rt)
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::suspend",
+                "the runtime system is not active (did you already call pika::stop?)");
+        }
+
+        rt->suspend();
+    }
+
+    void resume()
+    {
+        if (threads::detail::get_self_ptr())
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::resume",
+                "this function cannot be called from a pika thread");
+        }
+
+        runtime* rt = get_runtime_ptr();
+        if (nullptr == rt)
+        {
+            PIKA_THROW_EXCEPTION(pika::error::invalid_status, "pika::resume",
+                "the runtime system is not active (did you already call pika::stop?)");
+        }
+
+        rt->resume();
+    }
 }    // namespace pika
