@@ -7,7 +7,6 @@
 
 #include <pika/config.hpp>
 #include <pika/assert.hpp>
-#include <pika/futures/futures_factory.hpp>
 #include <pika/lock_registration/detail/register_locks.hpp>
 #include <pika/modules/debugging.hpp>
 #include <pika/modules/errors.hpp>
@@ -17,13 +16,16 @@
 #include <pika/runtime/config_entry.hpp>
 #include <pika/runtime/custom_exception_info.hpp>
 #include <pika/runtime/debugging.hpp>
-#include <pika/runtime/get_locality_id.hpp>
 #include <pika/runtime/get_worker_thread_num.hpp>
 #include <pika/runtime/runtime.hpp>
 #include <pika/runtime/state.hpp>
 #include <pika/threading_base/thread_helpers.hpp>
 #include <pika/version.hpp>
 
+#if defined(PIKA_HAVE_MPI)
+# include <pika/mpi_base/mpi.hpp>
+#endif
+
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 #include <fmt/printf.h>
@@ -85,8 +87,17 @@ namespace pika {
                 strm << "{stack-trace}: " << *back_trace << "\n";
             }
 
-            std::uint32_t const* locality = xi.get<pika::detail::throw_locality>();
-            if (locality) strm << "{locality-id}: " << *locality << "\n";
+#if defined(PIKA_HAVE_MPI)
+            int mpi_initialized = 0;
+            if (MPI_Initialized(&mpi_initialized) == MPI_SUCCESS && mpi_initialized)
+            {
+                int rank = 0;
+                if (MPI_Comm_rank(MPI_COMM_WORLD, &rank) == MPI_SUCCESS)
+                {
+                    strm << "{mpi-rank}: " << rank << '\n';
+                }
+            }
+#endif
 
             std::string const* hostname_ = xi.get<pika::detail::throw_hostname>();
             if (hostname_ && !hostname_->empty()) strm << "{hostname}: " << *hostname_ << "\n";
@@ -211,15 +222,14 @@ namespace pika {
         }
 
         pika::exception_info construct_exception_info(std::string const& func,
-            std::string const& file, long line, std::string const& back_trace, std::uint32_t node,
+            std::string const& file, long line, std::string const& back_trace,
             std::string const& hostname, std::int64_t pid, std::size_t shepherd,
             std::size_t thread_id, std::string const& thread_name, std::string const& env,
             std::string const& config, std::string const& state_name, std::string const& auxinfo)
         {
             return pika::exception_info().set(pika::detail::throw_stacktrace(back_trace),
-                pika::detail::throw_locality(node), pika::detail::throw_hostname(hostname),
-                pika::detail::throw_pid(pid), pika::detail::throw_shepherd(shepherd),
-                pika::detail::throw_thread_id(thread_id),
+                pika::detail::throw_hostname(hostname), pika::detail::throw_pid(pid),
+                pika::detail::throw_shepherd(shepherd), pika::detail::throw_thread_id(thread_id),
                 pika::detail::throw_thread_name(thread_name), pika::detail::throw_function(func),
                 pika::detail::throw_file(file), pika::detail::throw_line(line),
                 pika::detail::throw_env(env), pika::detail::throw_config(config),
@@ -352,14 +362,13 @@ namespace pika {
 
                 if (rts_state >= runtime_state::initialized && rts_state < runtime_state::stopped)
                 {
-                    hostname = get_runtime().here();
+                    hostname = pika::debug::detail::hostname_print_helper{}.get_hostname();
                 }
             }
 
             // if this is not a pika thread we do not need to query neither for
             // the shepherd thread nor for the thread id
             error_code ec(throwmode::lightweight);
-            std::uint32_t node = get_locality_id(ec);
 
             std::size_t shepherd = std::size_t(-1);
             threads::detail::thread_id_type thread_id;
@@ -379,8 +388,8 @@ namespace pika {
             std::string config(pika::configuration_string());
 
             return pika::exception_info().set(pika::detail::throw_stacktrace(back_trace),
-                pika::detail::throw_locality(node), pika::detail::throw_hostname(hostname),
-                pika::detail::throw_pid(pid), pika::detail::throw_shepherd(shepherd),
+                pika::detail::throw_hostname(hostname), pika::detail::throw_pid(pid),
+                pika::detail::throw_shepherd(shepherd),
                 pika::detail::throw_thread_id(reinterpret_cast<std::size_t>(thread_id.get())),
                 pika::detail::throw_thread_name(detail::as_string(thread_name)),
                 pika::detail::throw_function(func), pika::detail::throw_file(file),
@@ -393,7 +402,7 @@ namespace pika {
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace pika {
-    /// Return the host-name of the locality where the exception was thrown.
+    /// Return the host-name of the process where the exception was thrown.
     std::string get_error_host_name(pika::exception_info const& xi)
     {
         std::string const* hostname_ = xi.get<pika::detail::throw_hostname>();
@@ -401,17 +410,7 @@ namespace pika {
         return std::string();
     }
 
-    /// Return the locality where the exception was thrown.
-    std::uint32_t get_error_locality_id(pika::exception_info const& xi)
-    {
-        std::uint32_t const* locality = xi.get<pika::detail::throw_locality>();
-        if (locality) return *locality;
-
-        // same as naming::invalid_locality_id
-        return ~static_cast<std::uint32_t>(0);
-    }
-
-    /// Return the (operating system) process id of the locality where the
+    /// Return the (operating system) process id of the process where the
     /// exception was thrown.
     std::int64_t get_error_process_id(pika::exception_info const& xi)
     {
