@@ -34,6 +34,7 @@
 #include <pika/thread_support/set_thread_name.hpp>
 #include <pika/threading_base/external_timer.hpp>
 #include <pika/threading_base/scheduler_mode.hpp>
+#include <pika/threading_base/thread_helpers.hpp>
 #include <pika/topology/topology.hpp>
 #include <pika/version.hpp>
 
@@ -1191,12 +1192,9 @@ namespace pika {
     void runtime::wait_finalize()
     {
         std::unique_lock<std::mutex> l(mtx_);
-        while (!stop_done_)
-        {
-            LRT_(info).format("runtime: about to enter wait state");
-            wait_condition_.wait(l);
-            LRT_(info).format("runtime: exiting wait state");
-        }
+        LRT_(info).format("runtime: about to enter wait state");
+        wait_condition_.wait(l, [&] { return stop_done_; });
+        LRT_(info).format("runtime: exiting wait state");
     }
 
     void runtime::wait_helper(std::mutex& mtx, std::condition_variable& cond, bool& running)
@@ -1238,9 +1236,7 @@ namespace pika {
         // wait for the thread to run
         {
             std::unique_lock<std::mutex> lk(mtx);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (!running)    // -V776 // -V1044
-                cond.wait(lk);
+            cond.wait(lk, [&] { return running; });
         }
 
         // block main thread
@@ -1659,4 +1655,6 @@ namespace pika {
             return execution::detail::get_stack_size_enum_name(size_enum);
         }
     }    // namespace threads
+
+    bool is_runtime_initialized() noexcept { return get_runtime_ptr() != nullptr; }
 }    // namespace pika
