@@ -102,6 +102,7 @@ namespace pika::threads::detail {
                 ::pika::threads::detail::get_topology().get_machine_affinity_mask())
           , outside_numa_domain_masks_(init.num_queues_,
                 ::pika::threads::detail::get_topology().get_machine_affinity_mask())
+          , init_barrier(init.num_queues_)
         {
             ::pika::threads::detail::resize(
                 steals_in_numa_domain_, threads::detail::hardware_concurrency());
@@ -296,7 +297,7 @@ namespace pika::threads::detail {
             threads::detail::thread_id_ref_type& thrd,
             bool /*scheduler_mode::enable_stealing*/) override
         {
-            std::size_t queues_size = queues_.size();
+            [[maybe_unused]] std::size_t queues_size = queues_.size();
 
             {
                 PIKA_ASSERT(num_thread < queues_size);
@@ -314,6 +315,7 @@ namespace pika::threads::detail {
                 if (have_staged) return false;
             }
 
+#if !defined(PIKA_HAVE_THREAD_SANITIZER)
             if (!running) { return false; }
 
             bool numa_stealing = has_scheduler_mode(scheduler_mode::enable_stealing_numa);
@@ -401,6 +403,9 @@ namespace pika::threads::detail {
                     }
                 }
             }
+#else
+            PIKA_UNUSED(running);
+#endif
 
             return false;
         }
@@ -798,7 +803,11 @@ namespace pika::threads::detail {
 
             if (::pika::threads::detail::any(core_mask) && ::pika::threads::detail::any(node_mask))
             {
-                ::pika::threads::detail::set(steals_in_numa_domain_, num_pu);
+                {
+                    std::lock_guard l{init_mtx};
+                    ::pika::threads::detail::set(steals_in_numa_domain_, num_pu);
+                }
+
                 numa_domain_masks_[num_thread] = node_mask;
             }
 
@@ -816,10 +825,18 @@ namespace pika::threads::detail {
             bool numa_stealing = has_scheduler_mode(scheduler_mode::enable_stealing_numa);
             if (numa_stealing && ::pika::threads::detail::any(first_mask & core_mask))
             {
-                ::pika::threads::detail::set(steals_outside_numa_domain_, num_pu);
+                {
+                    std::lock_guard l{init_mtx};
+                    ::pika::threads::detail::set(steals_outside_numa_domain_, num_pu);
+                }
+
                 outside_numa_domain_masks_[num_thread] =
                     ::pika::threads::detail::not_(node_mask) & machine_mask;
             }
+
+            // Synchronize all worker threads so that the masks can be accessed without locks in the
+            // regular scheduling loop
+            init_barrier.wait();
         }
 
         void on_stop_thread(std::size_t num_thread) override
@@ -842,6 +859,9 @@ namespace pika::threads::detail {
         ::pika::threads::detail::mask_type steals_outside_numa_domain_;
         std::vector<::pika::threads::detail::mask_type> numa_domain_masks_;
         std::vector<::pika::threads::detail::mask_type> outside_numa_domain_masks_;
+
+        pika::concurrency::detail::barrier init_barrier;
+        std::mutex init_mtx;
     };
 }    // namespace pika::threads::detail
 
