@@ -56,8 +56,7 @@ void test_multiple_readers()
 
         {
             std::unique_lock<mutex_type> lk(unblocked_count_mutex);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (unblocked_count < number_of_threads) { unblocked_condition.wait(lk); }
+            unblocked_condition.wait(lk, [&] { return unblocked_count >= number_of_threads; });
         }
 
         CHECK_LOCKED_VALUE_EQUAL(unblocked_count_mutex, unblocked_count, number_of_threads);
@@ -144,8 +143,7 @@ void test_reader_blocks_writer()
 
         {
             std::unique_lock<mutex_type> lk(unblocked_count_mutex);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (unblocked_count < 1) { unblocked_condition.wait(lk); }
+            unblocked_condition.wait(lk, [&] { return unblocked_count >= 1; });
         }
 
         CHECK_LOCKED_VALUE_EQUAL(unblocked_count_mutex, unblocked_count, 1u);
@@ -208,8 +206,7 @@ void test_unlocking_writer_unblocks_all_readers()
 
         {
             std::unique_lock<mutex_type> lk(unblocked_count_mutex);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (unblocked_count < reader_count) { unblocked_condition.wait(lk); }
+            unblocked_condition.wait(lk, [&] { return unblocked_count >= reader_count; });
         }
 
         CHECK_LOCKED_VALUE_EQUAL(unblocked_count_mutex, unblocked_count, reader_count);
@@ -271,8 +268,7 @@ void test_unlocking_last_reader_only_unblocks_one_writer()
 
         {
             std::unique_lock<mutex_type> lk(unblocked_count_mutex);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (unblocked_count < reader_count) { unblocked_condition.wait(lk); }
+            unblocked_condition.wait(lk, [&] { return unblocked_count >= reader_count; });
         }
 
         std::this_thread::sleep_for(std::chrono::seconds(1));
@@ -284,8 +280,7 @@ void test_unlocking_last_reader_only_unblocks_one_writer()
 
         {
             std::unique_lock<mutex_type> lk(unblocked_count_mutex);
-            // NOLINTNEXTLINE(bugprone-infinite-loop)
-            while (unblocked_count < (reader_count + 1)) { unblocked_condition.wait(lk); }
+            unblocked_condition.wait(lk, [&] { return unblocked_count >= reader_count + 1; });
         }
 
         CHECK_LOCKED_VALUE_EQUAL(unblocked_count_mutex, unblocked_count, reader_count + 1);
