@@ -17,67 +17,47 @@ namespace pika::threads {
         /// As the name suggests, this option can be used to disable all other
         /// options.
         nothing_special = 0x000,
-        /// The scheduler will periodically call a provided callback function
-        /// from a special pika thread to enable performing background-work, for
-        /// instance driving networking progress or garbage-collect AGAS.
-        do_background_work = 0x001,
         /// The kernel priority of the os-thread driving the scheduler will be
         /// reduced below normal.
-        reduce_thread_priority = 0x002,
-        /// The scheduler will wait for some unspecified amount of time before
-        /// exiting the scheduling loop while being terminated to make sure no
-        /// other work is being scheduled during processing the shutdown
-        /// request.
-        delay_exit = 0x004,
-        /// Some schedulers have the capability to act as 'embedded'
-        /// schedulers. In this case it needs to periodically invoke a provided
-        /// callback into the outer scheduler more frequently than normal. This
-        /// option enables this behavior.
-        fast_idle_mode = 0x008,
+        reduce_thread_priority = 0x001,
         /// This option allows for the scheduler to dynamically increase and
         /// reduce the number of processing units it runs on. Setting this value
         /// not succeed for schedulers that do not support this functionality.
-        enable_elasticity = 0x010,
+        enable_elasticity = 0x002,
         /// This option allows schedulers that support work thread/stealing to
         /// enable/disable it
-        enable_stealing = 0x020,
+        enable_stealing = 0x004,
         /// This option allows schedulersthat support it to disallow stealing
         /// between numa domains
-        enable_stealing_numa = 0x040,
+        enable_stealing_numa = 0x008,
         /// This option tells schedulersthat support it to add tasks round
         /// robin to queues on each core
-        assign_work_round_robin = 0x080,
+        assign_work_round_robin = 0x010,
         /// This option tells schedulers that support it to add tasks round to
         /// the same core/queue that the parent task is running on
-        assign_work_thread_parent = 0x100,
+        assign_work_thread_parent = 0x020,
         /// This option tells schedulers that support it to always (try to)
         /// steal high priority tasks from other queues before finishing their
         /// own lower priority tasks
-        steal_high_priority_first = 0x200,
+        steal_high_priority_first = 0x040,
         /// This option tells schedulers that support it to steal tasks only
         /// when their local queues are empty
-        steal_after_local = 0x400,
+        steal_after_local = 0x080,
         /// This option allows for certain schedulers to explicitly disable
         /// exponential idle-back off
-        enable_idle_backoff = 0x0800,
+        enable_idle_backoff = 0x100,
 
         // clang-format off
         /// This option represents the default mode.
         default_mode =
-            do_background_work |
             reduce_thread_priority |
-            delay_exit |
             enable_stealing |
             enable_stealing_numa |
             assign_work_round_robin |
-            steal_after_local |
-            enable_idle_backoff,
+            steal_after_local,
         /// This enables all available options.
         all_flags =
-            do_background_work |
             reduce_thread_priority |
-            delay_exit |
-            fast_idle_mode |
             enable_elasticity |
             enable_stealing |
             enable_stealing_numa |
